package org.jastadd.ros2rag.scanner;

import org.jastadd.ros2rag.parser.Ros2RagParser.Terminals;

%%

%public
%final
%class Ros2RagScanner
%extends beaver.Scanner

%type beaver.Symbol
%function nextToken
%yylexthrow beaver.Scanner.Exception
%scanerror Ros2RagScanner.ScannerError

%line
%column
%{
  private StringBuilder stringLitSb = new StringBuilder();

  private beaver.Symbol sym(short id) {
    return new beaver.Symbol(id, yyline + 1, yycolumn + 1, yylength(), yytext());
  }

  private beaver.Symbol sym(short id, String text) {
    return new beaver.Symbol(id, yyline + 1, yycolumn + 1, yylength(), text);
  }


  public static class ScannerError extends Error {
    public ScannerError(String message) {
      super(message);
    }
  }
%}

WhiteSpace = [ ] | \t | \f | \n | \r | \r\n
TraditionalComment   = [/][*][^*]*[*]+([^*/][^*]*[*]+)*[/]
EndOfLineComment = "//" [^\n\r]*
Comment = {TraditionalComment} | {EndOfLineComment}

MappingContent = [{][:][^:]*[:]+([^:}][^:]*[:]+)*[}]

ID = [a-zA-Z$_][a-zA-Z0-9$_]*

%%
{WhiteSpace} { /* ignore */ }
{Comment}    { return sym(Terminals.COMMENT); }
{MappingContent} { return sym(Terminals.MAPPING_CONTENT); }

"abstract"   { return sym(Terminals.ABSTRACT); }
"rel"        { return sym(Terminals.RELATION); }

"read"       { return sym(Terminals.READ); }
"write"      { return sym(Terminals.WRITE); }
"using"      { return sym(Terminals.USING); }
"canDependOn" { return sym(Terminals.CAN_DEPEND_ON); }
"maps"       { return sym(Terminals.MAPS); }
"to"         { return sym(Terminals.TO); }
"as"         { return sym(Terminals.AS); }
"with"       { return sym(Terminals.WITH); }

";"          { return sym(Terminals.SCOL); }
":"          { return sym(Terminals.COL); }
"::="        { return sym(Terminals.ASSIGN); }
"*"          { return sym(Terminals.STAR); }
"."          { return sym(Terminals.DOT); }
","          { return sym(Terminals.COMMA); }
"<"          { return sym(Terminals.LT); }
">"          { return sym(Terminals.GT); }
"["          { return sym(Terminals.LBRACKET); }
"]"          { return sym(Terminals.RBRACKET); }
"/"          { return sym(Terminals.SLASH); }
"?"          { return sym(Terminals.QUESTION_MARK); }
"->"         { return sym(Terminals.RIGHT); }
"<-"         { return sym(Terminals.LEFT); }
"<->"        { return sym(Terminals.BIDIRECTIONAL); }

// ID
{ID}         { return sym(Terminals.ID); }
<<EOF>>      { return sym(Terminals.EOF); }

[^]            { throw new ScannerError((yyline+1) +"," + (yycolumn+1) + ": Illegal character <"+yytext()+">"); }
